import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'config.m.dart';

class ConfigService extends JsonRpcClientBase {
  ConfigService(
    String host, {
    String serviceName = "IConfigService",
    Map<String, String>? headers,
    int? timeout,
  }) : super(
          host,
          serviceName,
          headers: headers,
          timeout: timeout,
        ) {
    /// 注册响应实体反序列化处理器
    FJsonConvert.setDecoder((map) => ConfigItem.fromJson(map));
  }

  Future<List<ConfigItem>> getConfigListAsync(PageParams pageParams) async {
    var rpcRst = await call("GetConfigListAsync", pageParams);
    var result = (rpcRst as List)
        .map((e) => ConfigItem.fromJson(e as Map<String, dynamic>))
        .toList();
    return result;
  }

  Future<String> getConfigAsync(String service, String section,
      String settingNode, String defaultValue) async {
    var rpcRst = await call(
        "GetConfigAsync", [service, section, settingNode, defaultValue]);
    return rpcRst;
  }

  Future<bool> setConfigAsync(String service, String section,
      String settingNode, String settingValue) async {
    var rpcRst = await call(
        "SetConfigAsync", [service, section, settingNode, settingValue]);
    return rpcRst;
  }
}
