import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'department.m.dart';

class DepartmentService extends JsonRpcClientBase {
	DepartmentService(
		String host, {
		String serviceName = "IDepartmentService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => DepartmentInfo.fromJson(map));
		FJsonConvert.setDecoder((map) => PageCollection<DepartmentInfo>.fromJson(map));
	}

	Future<String> insertDepartmentAsync(DepartmentInfo adminDO,String extensionData) async {
		var rpcRst = await call("InsertDepartmentAsync", [adminDO,extensionData]);
		return rpcRst;
	}

	Future<DepartmentInfo> findDepartmentByNameAsync(String name) async {
		var rpcRst = await call("FindDepartmentByNameAsync", name);
		var result = DepartmentInfo.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<DepartmentInfo> findDepartmentByCodeAsync(String code) async {
		var rpcRst = await call("FindDepartmentByCodeAsync", code);
		var result = DepartmentInfo.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<PageCollection<DepartmentInfo>> findDepartmentPagesAsync(PageRequest request) async {
		var rpcRst = await call("FindDepartmentPagesAsync", request);
		var result = PageCollection<DepartmentInfo>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> deleteDepartmentByIdAsync(String id) async {
		var rpcRst = await call("DeleteDepartmentByIdAsync", id);
		return rpcRst;
	}

	Future<bool> updateDepartmentByIdAsync(String id,DepartmentInfo data) async {
		var rpcRst = await call("UpdateDepartmentByIdAsync", [id,data]);
		return rpcRst;
	}

	Future<bool> updateDepartmentByCodeAsync(String code,DepartmentInfo data) async {
		var rpcRst = await call("UpdateDepartmentByCodeAsync", [code,data]);
		return rpcRst;
	}

}

