import 'package:fis_common/json_convert.dart';

class DepartmentInfo {
	String? departmentCode;
	String? departmentName;
	String? organizationCode;

	DepartmentInfo({
		this.departmentCode,
		this.departmentName,
		this.organizationCode,
	});

	factory DepartmentInfo.fromJson(Map<String, dynamic> map) {
		return DepartmentInfo( 
			departmentCode: map['DepartmentCode'],
			departmentName: map['DepartmentName'],
			organizationCode: map['OrganizationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(departmentCode != null)
			map['DepartmentCode'] = departmentCode;
		if(departmentName != null)
			map['DepartmentName'] = departmentName;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		return map;
	}
}

class PageCollection<T> {
	int currentPage;
	int pageSize;
	int dataCount;
	List<T>? pageData;

	PageCollection({
		this.currentPage=0,
		this.pageSize=0,
		this.dataCount=0,
		this.pageData,
	});

	factory PageCollection.fromJson(Map<String, dynamic> map) {
		List<T> pageDataList = [];
		if (map['PageData'] != null) {
			pageDataList.addAll(
					(map['PageData'] as List).map((e) => FJsonConvert.fromJson<T>(e)!));
		}
		return PageCollection( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			dataCount: map['DataCount'],
			pageData: pageDataList,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		map['DataCount'] = dataCount;
		if(pageData != null)
			map['PageData'] = pageData;
		return map;
	}
}

class PageRequest {
	int currentPage;
	int pageSize;
	Map<String,String>? filter;
	bool isFuzzy;
	String? sessionId;

	PageRequest({
		this.currentPage=0,
		this.pageSize=0,
		this.filter,
		this.isFuzzy=false,
		this.sessionId,
	});

	factory PageRequest.fromJson(Map<String, dynamic> map) {
		return PageRequest( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			filter: map['Filter'].cast<String,String>(),
			isFuzzy: map['IsFuzzy'],
			sessionId: map['SessionId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		if(filter != null)
			map['Filter'] = filter;
		map['IsFuzzy'] = isFuzzy;
		if(sessionId != null)
			map['SessionId'] = sessionId;
		return map;
	}
}


