import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'device.m.dart';

class DeviceService extends JsonRpcClientBase {
	DeviceService(
		String host, {
		String serviceName = "IDeviceService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => DeviceInfo.fromJson(map));
		FJsonConvert.setDecoder((map) => PageCollection<DeviceInfo>.fromJson(map));
	}

	Future<String> createDeviceInfoAsync(CreateDeviceInfoRequest request) async {
		var rpcRst = await call("CreateDeviceInfoAsync", request);
		return rpcRst;
	}

	Future<DeviceInfo> getDeviceInfoAsync(String deviceCode) async {
		var rpcRst = await call("GetDeviceInfoAsync", deviceCode);
		var result = DeviceInfo.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<PageCollection<DeviceInfo>> getDeviceInfoPageAsync(PageRequest request) async {
		var rpcRst = await call("GetDeviceInfoPageAsync", request);
		var result = PageCollection<DeviceInfo>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

}

