import 'package:fis_jsonrpc/utils.dart';

import 'package:fis_common/json_convert.dart';

class BaseRequest {
	String? token;

	BaseRequest({
		this.token,
	});

	factory BaseRequest.fromJson(Map<String, dynamic> map) {
		return BaseRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(token != null)
			map['Token'] = token;
		return map;
	}
}

class BaseEntity {
	DateTime? createTime;
	DateTime? updateTime;

	BaseEntity({
		this.createTime,
		this.updateTime,
	});

	factory BaseEntity.fromJson(Map<String, dynamic> map) {
		return BaseEntity( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(updateTime != null)
			map['UpdateTime'] = JsonRpcUtils.dateFormat(updateTime!);
		return map;
	}
}

class DeviceInfo extends BaseEntity{
	String? deviceCode;
	String? serialNumber;
	String? description;
	String? deviceModel;
	String? deviceType;
	String? headPicUrl;
	String? deviceSoftwareVersion;
	String? sDKSoftwareVersion;
	String? organizationCode;
	String? shortCode;

	DeviceInfo({
		this.deviceCode,
		this.serialNumber,
		this.description,
		this.deviceModel,
		this.deviceType,
		this.headPicUrl,
		this.deviceSoftwareVersion,
		this.sDKSoftwareVersion,
		this.organizationCode,
		this.shortCode,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory DeviceInfo.fromJson(Map<String, dynamic> map) {
		return DeviceInfo( 
			deviceCode: map['DeviceCode'],
			serialNumber: map['SerialNumber'],
			description: map['Description'],
			deviceModel: map['DeviceModel'],
			deviceType: map['DeviceType'],
			headPicUrl: map['HeadPicUrl'],
			deviceSoftwareVersion: map['DeviceSoftwareVersion'],
			sDKSoftwareVersion: map['SDKSoftwareVersion'],
			organizationCode: map['OrganizationCode'],
			shortCode: map['ShortCode'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(serialNumber != null)
			map['SerialNumber'] = serialNumber;
		if(description != null)
			map['Description'] = description;
		if(deviceModel != null)
			map['DeviceModel'] = deviceModel;
		if(deviceType != null)
			map['DeviceType'] = deviceType;
		if(headPicUrl != null)
			map['HeadPicUrl'] = headPicUrl;
		if(deviceSoftwareVersion != null)
			map['DeviceSoftwareVersion'] = deviceSoftwareVersion;
		if(sDKSoftwareVersion != null)
			map['SDKSoftwareVersion'] = sDKSoftwareVersion;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(shortCode != null)
			map['ShortCode'] = shortCode;
		return map;
	}
}

class CreateDeviceInfoRequest extends BaseRequest{
	DeviceInfo? info;

	CreateDeviceInfoRequest({
		this.info,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateDeviceInfoRequest.fromJson(Map<String, dynamic> map) {
		return CreateDeviceInfoRequest( 
			info: map['Info'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(info != null)
			map['Info'] = info;
		return map;
	}
}

class PageCollection<T> {
	int currentPage;
	int pageSize;
	int dataCount;
	List<T>? pageData;

	PageCollection({
		this.currentPage = 0,
		this.pageSize = 0,
		this.dataCount = 0,
		this.pageData,
	});

	factory PageCollection.fromJson(Map<String, dynamic> map) {
		List<T> pageDataList = [];
		if (map['PageData'] != null) {
			pageDataList.addAll(
					(map['PageData'] as List).map((e) => FJsonConvert.fromJson<T>(e)!));
		}
		return PageCollection( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			dataCount: map['DataCount'],
			pageData: pageDataList,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		map['DataCount'] = dataCount;
		if(pageData != null)
			map['PageData'] = pageData;
		return map;
	}
}

class PageRequest extends BaseRequest{
	int currentPage;
	int pageSize;
	Map<String,String>? filter;
	bool isFuzzy;

	PageRequest({
		this.currentPage = 0,
		this.pageSize = 0,
		this.filter,
		this.isFuzzy = false,
		String? token,
	}) : super(
			token: token,
		);

	factory PageRequest.fromJson(Map<String, dynamic> map) {
		return PageRequest( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			filter: map['Filter'].cast<String,String>(),
			isFuzzy: map['IsFuzzy'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		if(filter != null)
			map['Filter'] = filter;
		map['IsFuzzy'] = isFuzzy;
		return map;
	}
}



