import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'rank.m.dart';

class RankService extends JsonRpcClientBase {
	RankService(
		String host, {
		String serviceName = "IRankService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => RankDTO.fromJson(map));
	}

	Future<RankDTO> getRankByCode(GetRankByCodeRequest request) async {
		var rpcRst = await call("GetRankByCode", request);
		var result = RankDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<RankDTO>> getAllRanks(GetRanksRequest request) async {
		var rpcRst = await call("GetAllRanks", request);
		var result = (rpcRst as List).map((e)=>RankDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<int> addRanks(AddRanksRequest request) async {
		var rpcRst = await call("AddRanks", request);
		return rpcRst;
	}

	Future<int> removeRanks(RemoveRanksRequest request) async {
		var rpcRst = await call("RemoveRanks", request);
		return rpcRst;
	}

	Future<int> updateRanks(UpdateRanksRequest request) async {
		var rpcRst = await call("UpdateRanks", request);
		return rpcRst;
	}

}

