import 'package:fis_jsonrpc/utils.dart';

class BaseDTO {
  DateTime? createTime;
  DateTime? updateTime;

  BaseDTO({
    this.createTime,
    this.updateTime,
  });

  factory BaseDTO.fromJson(Map<String, dynamic> map) {
    return BaseDTO(
      createTime:
          map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
      updateTime:
          map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    if (createTime != null)
      map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
    if (updateTime != null)
      map['UpdateTime'] = JsonRpcUtils.dateFormat(updateTime!);
    return map;
  }
}

class RankDTO extends BaseDTO {
  String? rankCode;
  String? rankName;

  RankDTO({
    this.rankCode,
    this.rankName,
    DateTime? createTime,
    DateTime? updateTime,
  }) : super(
          createTime: createTime,
          updateTime: updateTime,
        );

  factory RankDTO.fromJson(Map<String, dynamic> map) {
    return RankDTO(
      rankCode: map['RankCode'],
      rankName: map['RankName'],
      createTime:
          map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
      updateTime:
          map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
    );
  }

  Map<String, dynamic> toJson() {
    final map = super.toJson();
    if (rankCode != null) map['RankCode'] = rankCode;
    if (rankName != null) map['RankName'] = rankName;
    return map;
  }
}

class BaseRequest {
  BaseRequest();

  factory BaseRequest.fromJson(Map<String, dynamic> map) {
    return BaseRequest();
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    return map;
  }
}

class TokenRequest extends BaseRequest {
  String? token;

  TokenRequest({
    this.token,
  }) : super();

  factory TokenRequest.fromJson(Map<String, dynamic> map) {
    return TokenRequest(
      token: map['Token'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = super.toJson();
    if (token != null) map['Token'] = token;
    return map;
  }
}

class GetRankByCodeRequest extends TokenRequest {
  String? rankCode;

  GetRankByCodeRequest({
    this.rankCode,
    String? token,
  }) : super(
          token: token,
        );

  factory GetRankByCodeRequest.fromJson(Map<String, dynamic> map) {
    return GetRankByCodeRequest(
      rankCode: map['RankCode'],
      token: map['Token'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = super.toJson();
    if (rankCode != null) map['RankCode'] = rankCode;
    return map;
  }
}

class GetRanksRequest extends TokenRequest {
  List<String>? rankCodes;
  List<String>? organizationCodes;

  GetRanksRequest({
    this.rankCodes,
    this.organizationCodes,
    String? token,
  }) : super(
          token: token,
        );

  factory GetRanksRequest.fromJson(Map<String, dynamic> map) {
    return GetRanksRequest(
      rankCodes: map['RankCodes'].cast<String>().toList(),
      organizationCodes: map['OrganizationCodes'].cast<String>().toList(),
      token: map['Token'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = super.toJson();
    if (rankCodes != null) map['RankCodes'] = rankCodes;
    if (organizationCodes != null) map['OrganizationCodes'] = organizationCodes;
    return map;
  }
}

class RankItemDTO {
  String? rankName;
  String? organizationCode;
  String? extendsData;

  RankItemDTO({
    this.rankName,
    this.organizationCode,
    this.extendsData,
  });

  factory RankItemDTO.fromJson(Map<String, dynamic> map) {
    return RankItemDTO(
      rankName: map['RankName'],
      organizationCode: map['OrganizationCode'],
      extendsData: map['ExtendsData'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    if (rankName != null) map['RankName'] = rankName;
    if (organizationCode != null) map['OrganizationCode'] = organizationCode;
    if (extendsData != null) map['ExtendsData'] = extendsData;
    return map;
  }
}

class AddRanksRequest extends TokenRequest {
  List<RankItemDTO>? rankInfos;

  AddRanksRequest({
    this.rankInfos,
    String? token,
  }) : super(
          token: token,
        );

  factory AddRanksRequest.fromJson(Map<String, dynamic> map) {
    return AddRanksRequest(
      rankInfos: map['RankInfos']
          .map((e) => RankItemDTO.fromJson(e as Map<String, dynamic>))
          .toList(),
      token: map['Token'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = super.toJson();
    if (rankInfos != null) map['RankInfos'] = rankInfos;
    return map;
  }
}

class RemoveRanksRequest extends TokenRequest {
  List<String>? rankCodes;

  RemoveRanksRequest({
    this.rankCodes,
    String? token,
  }) : super(
          token: token,
        );

  factory RemoveRanksRequest.fromJson(Map<String, dynamic> map) {
    return RemoveRanksRequest(
      rankCodes: map['RankCodes'].cast<String>().toList(),
      token: map['Token'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = super.toJson();
    if (rankCodes != null) map['RankCodes'] = rankCodes;
    return map;
  }
}

class UpdateRanksRequest extends TokenRequest {
  List<RankItemDTO>? ranks;

  UpdateRanksRequest({
    this.ranks,
    String? token,
  }) : super(
          token: token,
        );

  factory UpdateRanksRequest.fromJson(Map<String, dynamic> map) {
    return UpdateRanksRequest(
      ranks: map['Ranks']
          .map((e) => RankItemDTO.fromJson(e as Map<String, dynamic>))
          .toList(),
      token: map['Token'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = super.toJson();
    if (ranks != null) map['Ranks'] = ranks;
    return map;
  }
}
