import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'remedical.m.dart';

class RemedicalService extends JsonRpcClientBase {
	RemedicalService(
		String host, {
		String serviceName = "IRemedicalService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => DeviceData.fromJson(map));
		FJsonConvert.setDecoder((map) => RecordInfo.fromJson(map));
		FJsonConvert.setDecoder((map) => PageCollection<RecordInfo>.fromJson(map));
		FJsonConvert.setDecoder((map) => PatientInfo.fromJson(map));
	}

	Future<String> createPatientInfoAsync(CreatePatientInfoRequest request) async {
		var rpcRst = await call("CreatePatientInfoAsync", request);
		return rpcRst;
	}

	Future<String> createDeviceDataAsync(CreateDeviceDataRequest request) async {
		var rpcRst = await call("CreateDeviceDataAsync", request);
		return rpcRst;
	}

	Future<String> createRecordInfoAsync(CreateRecordInfoRequest request) async {
		var rpcRst = await call("CreateRecordInfoAsync", request);
		return rpcRst;
	}

	Future<DeviceData> getDeviceDataDetailAsync(GetDeviceDataDetailRequest request) async {
		var rpcRst = await call("GetDeviceDataDetailAsync", request);
		var result = DeviceData.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<RecordInfo> getRecordInfoDetailAsync(GetRecordInfoDetailRequest request) async {
		var rpcRst = await call("GetRecordInfoDetailAsync", request);
		var result = RecordInfo.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<PageCollection<RecordInfo>> getRecordInfoPageAsync(PageRequest request) async {
		var rpcRst = await call("GetRecordInfoPageAsync", request);
		var result = PageCollection<RecordInfo>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<PatientInfo> getPatientInfoAsync(GetPatientInfoRequest request) async {
		var rpcRst = await call("GetPatientInfoAsync", request);
		var result = PatientInfo.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

}

