import 'package:fis_jsonrpc/utils.dart';

import 'package:fis_common/json_convert.dart';

class BaseRequest {
	String? token;

	BaseRequest({
		this.token,
	});

	factory BaseRequest.fromJson(Map<String, dynamic> map) {
		return BaseRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(token != null)
			map['Token'] = token;
		return map;
	}
}

class BaseEntity {
	DateTime? createTime;
	DateTime? updateTime;

	BaseEntity({
		this.createTime,
		this.updateTime,
	});

	factory BaseEntity.fromJson(Map<String, dynamic> map) {
		return BaseEntity( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(updateTime != null)
			map['UpdateTime'] = JsonRpcUtils.dateFormat(updateTime!);
		return map;
	}
}

enum GenderTypeEnum {
	Male,
	Female,
}

enum PatientTypeEnum {
	Default,
}

class PatientInfo extends BaseEntity{
	String? patientCode;
	String? firstName;
	String? lastName;
	String? patientCardNo;
	DateTime? birthday;
	GenderTypeEnum genderType;
	String? patientCaseHistory;
	String? patientPhone;
	PatientTypeEnum patientType;

	PatientInfo({
		this.patientCode,
		this.firstName,
		this.lastName,
		this.patientCardNo,
		this.birthday,
		this.genderType = GenderTypeEnum.Male,
		this.patientCaseHistory,
		this.patientPhone,
		this.patientType = PatientTypeEnum.Default,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory PatientInfo.fromJson(Map<String, dynamic> map) {
		return PatientInfo( 
			patientCode: map['PatientCode'],
			firstName: map['FirstName'],
			lastName: map['LastName'],
			patientCardNo: map['PatientCardNo'],
			birthday: map['Birthday'] != null ? DateTime.parse(map['Birthday']) : null,
			genderType: GenderTypeEnum.values.firstWhere((e) => e.index == map['GenderType']),
			patientCaseHistory: map['PatientCaseHistory'],
			patientPhone: map['PatientPhone'],
			patientType: PatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		if(firstName != null)
			map['FirstName'] = firstName;
		if(lastName != null)
			map['LastName'] = lastName;
		if(patientCardNo != null)
			map['PatientCardNo'] = patientCardNo;
		if(birthday != null)
			map['Birthday'] = JsonRpcUtils.dateFormat(birthday!);
		map['GenderType'] = genderType.index;
		if(patientCaseHistory != null)
			map['PatientCaseHistory'] = patientCaseHistory;
		if(patientPhone != null)
			map['PatientPhone'] = patientPhone;
		map['PatientType'] = patientType.index;
		return map;
	}
}

class CreatePatientInfoRequest extends BaseRequest{
	PatientInfo? info;

	CreatePatientInfoRequest({
		this.info,
		String? token,
	}) : super(
			token: token,
		);

	factory CreatePatientInfoRequest.fromJson(Map<String, dynamic> map) {
		return CreatePatientInfoRequest( 
			info: map['Info'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(info != null)
			map['Info'] = info;
		return map;
	}
}

enum DeviceDataTypeEnum {
	Default,
}

class DeviceData extends BaseEntity{
	String? deviceDataCode;
	String? deviceCode;
	String? deviceFileCode;
	String? recordCode;
	String? patientCode;
	String? previewImageToken;
	String? dataToken;
	DeviceDataTypeEnum deviceDataType;
	String? processResult;

	DeviceData({
		this.deviceDataCode,
		this.deviceCode,
		this.deviceFileCode,
		this.recordCode,
		this.patientCode,
		this.previewImageToken,
		this.dataToken,
		this.deviceDataType = DeviceDataTypeEnum.Default,
		this.processResult,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory DeviceData.fromJson(Map<String, dynamic> map) {
		return DeviceData( 
			deviceDataCode: map['DeviceDataCode'],
			deviceCode: map['DeviceCode'],
			deviceFileCode: map['DeviceFileCode'],
			recordCode: map['RecordCode'],
			patientCode: map['PatientCode'],
			previewImageToken: map['PreviewImageToken'],
			dataToken: map['DataToken'],
			deviceDataType: DeviceDataTypeEnum.values.firstWhere((e) => e.index == map['DeviceDataType']),
			processResult: map['ProcessResult'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceDataCode != null)
			map['DeviceDataCode'] = deviceDataCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(deviceFileCode != null)
			map['DeviceFileCode'] = deviceFileCode;
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		if(previewImageToken != null)
			map['PreviewImageToken'] = previewImageToken;
		if(dataToken != null)
			map['DataToken'] = dataToken;
		map['DeviceDataType'] = deviceDataType.index;
		if(processResult != null)
			map['ProcessResult'] = processResult;
		return map;
	}
}

class CreateDeviceDataRequest extends BaseRequest{
	DeviceData? info;

	CreateDeviceDataRequest({
		this.info,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateDeviceDataRequest.fromJson(Map<String, dynamic> map) {
		return CreateDeviceDataRequest( 
			info: map['Info'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(info != null)
			map['Info'] = info;
		return map;
	}
}

enum RecordTypeEnum {
	Ultrasound,
	Electrocardio,
}

enum CheckTypeEnum {
	Default,
}

enum RecordStatusEnum {
	Default,
}

class RecordInfo extends BaseEntity{
	String? recordCode;
	String? patientCode;
	String? patientName;
	String? orgName;
	RecordTypeEnum recordType;
	CheckTypeEnum checkType;
	String? localRecordCode;
	RecordStatusEnum recordStatus;
	String? recordRemark;
	String? tags;

	RecordInfo({
		this.recordCode,
		this.patientCode,
		this.patientName,
		this.orgName,
		this.recordType = RecordTypeEnum.Ultrasound,
		this.checkType = CheckTypeEnum.Default,
		this.localRecordCode,
		this.recordStatus = RecordStatusEnum.Default,
		this.recordRemark,
		this.tags,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory RecordInfo.fromJson(Map<String, dynamic> map) {
		return RecordInfo( 
			recordCode: map['RecordCode'],
			patientCode: map['PatientCode'],
			patientName: map['PatientName'],
			orgName: map['OrgName'],
			recordType: RecordTypeEnum.values.firstWhere((e) => e.index == map['RecordType']),
			checkType: CheckTypeEnum.values.firstWhere((e) => e.index == map['CheckType']),
			localRecordCode: map['LocalRecordCode'],
			recordStatus: RecordStatusEnum.values.firstWhere((e) => e.index == map['RecordStatus']),
			recordRemark: map['RecordRemark'],
			tags: map['Tags'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(orgName != null)
			map['OrgName'] = orgName;
		map['RecordType'] = recordType.index;
		map['CheckType'] = checkType.index;
		if(localRecordCode != null)
			map['LocalRecordCode'] = localRecordCode;
		map['RecordStatus'] = recordStatus.index;
		if(recordRemark != null)
			map['RecordRemark'] = recordRemark;
		if(tags != null)
			map['Tags'] = tags;
		return map;
	}
}

class CreateRecordInfoRequest extends BaseRequest{
	RecordInfo? info;

	CreateRecordInfoRequest({
		this.info,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateRecordInfoRequest.fromJson(Map<String, dynamic> map) {
		return CreateRecordInfoRequest( 
			info: map['Info'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(info != null)
			map['Info'] = info;
		return map;
	}
}

class GetDeviceDataDetailRequest extends BaseRequest{
	String? deviceDataCode;

	GetDeviceDataDetailRequest({
		this.deviceDataCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetDeviceDataDetailRequest.fromJson(Map<String, dynamic> map) {
		return GetDeviceDataDetailRequest( 
			deviceDataCode: map['DeviceDataCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceDataCode != null)
			map['DeviceDataCode'] = deviceDataCode;
		return map;
	}
}

class GetRecordInfoDetailRequest extends BaseRequest{
	String? recordCode;

	GetRecordInfoDetailRequest({
		this.recordCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetRecordInfoDetailRequest.fromJson(Map<String, dynamic> map) {
		return GetRecordInfoDetailRequest( 
			recordCode: map['RecordCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		return map;
	}
}

class PageCollection<T> {
	int currentPage;
	int pageSize;
	int dataCount;
	List<T>? pageData;

	PageCollection({
		this.currentPage = 0,
		this.pageSize = 0,
		this.dataCount = 0,
		this.pageData,
	});

	factory PageCollection.fromJson(Map<String, dynamic> map) {
		List<T> pageDataList = [];
		if (map['PageData'] != null) {
			pageDataList.addAll(
					(map['PageData'] as List).map((e) => FJsonConvert.fromJson<T>(e)!));
		}
		return PageCollection( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			dataCount: map['DataCount'],
			pageData: pageDataList,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		map['DataCount'] = dataCount;
		if(pageData != null)
			map['PageData'] = pageData;
		return map;
	}
}

class PageRequest extends BaseRequest{
	int currentPage;
	int pageSize;
	Map<String,String>? filter;
	bool isFuzzy;

	PageRequest({
		this.currentPage = 0,
		this.pageSize = 0,
		this.filter,
		this.isFuzzy = false,
		String? token,
	}) : super(
			token: token,
		);

	factory PageRequest.fromJson(Map<String, dynamic> map) {
		return PageRequest( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			filter: map['Filter'].cast<String,String>(),
			isFuzzy: map['IsFuzzy'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		if(filter != null)
			map['Filter'] = filter;
		map['IsFuzzy'] = isFuzzy;
		return map;
	}
}

class GetPatientInfoRequest extends BaseRequest{
	String? patientCode;

	GetPatientInfoRequest({
		this.patientCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetPatientInfoRequest.fromJson(Map<String, dynamic> map) {
		return GetPatientInfoRequest( 
			patientCode: map['PatientCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		return map;
	}
}



