import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'role.m.dart';

class RoleService extends JsonRpcClientBase {
	RoleService(
		String host, {
		String serviceName = "IRoleService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => RoleDTO.fromJson(map));
	}

	Future<RoleDTO> getRoleByCode(GetRoleByCodeRequest request) async {
		var rpcRst = await call("GetRoleByCode", request);
		var result = RoleDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<RoleDTO>> findDefaultRoles(FindDefaultRolesRequest request) async {
		var rpcRst = await call("FindDefaultRoles", request);
		var result = (rpcRst as List).map((e)=>RoleDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<List<RoleDTO>> findAuthenticationRoles(FindAuthenticationRolesRequest request) async {
		var rpcRst = await call("FindAuthenticationRoles", request);
		var result = (rpcRst as List).map((e)=>RoleDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

}

